use rug::Integer;
use std::fs;
use std::io;
use std::path::Path;
use std::str::FromStr;

pub struct AgcdInput {
    pub noise_bits: usize,
    pub numbers: Vec<Integer>,
}

// Parse the input file, return noise_bits and numbers.
pub fn parse_file(path: &Path) -> io::Result<AgcdInput> {
    let content = fs::read_to_string(path)?;
    let mut lines = content.lines();

    // Parse noise_bits from first non comment line
    let noise_bits = loop {
        match lines.next() {
            Some(first_line) => {
                let trimmed = first_line.trim();
                if trimmed.starts_with("//") {
                    continue; // Skip comment lines
                }
                break trimmed.parse::<usize>().map_err(|e| {
                    io::Error::new(
                        io::ErrorKind::InvalidData,
                        format!("First non-comment line must be noise_bits (usize): {}", e),
                    )
                })?;
            }
            None => {
                return Err(io::Error::new(
                    io::ErrorKind::InvalidData,
                    "Input file is empty or contains only comments",
                ))
            }
        }
    };

    // Parse numbers from remaining lines
    let numbers: Vec<Integer> = lines
        .filter(|line| {
            let t = line.trim();
            !t.starts_with("//") && !t.is_empty()
        })
        .map(|line| {
            let s = line.trim();
            Integer::from_str(s).map_err(|e| {
                io::Error::new(
                    io::ErrorKind::InvalidData,
                    format!("Failed to parse integer '{}': {}", s, e),
                )
            })
        })
        .collect::<Result<_, io::Error>>()?;

    if numbers.len() < 2 {
        return Err(io::Error::new(
            io::ErrorKind::InvalidData,
            "Need at least 2 numbers to compute AGCD",
        ));
    }

    Ok(AgcdInput {
        noise_bits,
        numbers,
    })
}
